@@ -286,14 +286,12 @@ def _inject_concurrency_blockers(items, node_os, node_os_version):
     for item_type in item_types:
         block_concurrent = [item_type.ITEM_TYPE_NAME]
         block_concurrent.extend(item_type.block_concurrent(node_os, node_os_version))
-        found = False
         for blocked_types in chain_groups:
             for blocked_type in block_concurrent:
                 if blocked_type in blocked_types:
                     blocked_types.extend(block_concurrent)
-                    found = True
                     break
-        if not found:
+        else:
             chain_groups.append(block_concurrent)
 
     # daisy-chain all items of the chain group while respecting existing
@@ -401,6 +399,9 @@ def _inject_reverse_dependencies(items):
             if depending_item_id.startswith("bundle:"):
                 depending_bundle_name = depending_item_id.split(":")[1]
                 for depending_item in items.values():
+                    if isinstance(depending_item, DummyItem):
+                        # Type and TagItems don't have bundles
+                        continue
                     if depending_item.bundle.name == depending_bundle_name:
                         add_dep(depending_item, item.id)
 
