@@ -105,6 +105,8 @@ def content_processor_mako(item):
             # error message - even though we can't pinpoint the excat
             # location of the error. :/
             e = _("Undefined variable (look for '${...}')")
+        elif isinstance(e, KeyError):
+            e = _("KeyError: {}").format(str(e))
         raise TemplateError(_(
             "Error while rendering template for {node}:{bundle}:{item}: {error}"
         ).format(
@@ -188,14 +190,10 @@ class File(Item):
     def _template_content(self):
         if self.attributes['source'] is not None:
             filename = join(self.item_data_dir, self.attributes['source'])
-            if exists(filename):
-                with open(filename, 'rb') as f:
-                    content = f.read()
-            else:
+            if not exists(filename):
                 filename = join(self.item_dir, self.attributes['source'])
-                with open(filename, 'rb') as f:
-                    content = f.read()
-            return force_text(content)
+            with open(filename, 'rb') as f:
+                return force_text(f.read())
         else:
             return force_text(self.attributes['content'])
 
@@ -378,6 +376,14 @@ class File(Item):
             attributes['group'] = 'wheel'
         return attributes
 
+    def preview(self):
+        if (
+            self.attributes['content_type'] in ('any', 'base64', 'binary') or
+            self.attributes['delete'] is True
+        ):
+            raise ValueError
+        return self.content.decode(self.attributes['encoding'])
+
     def test(self):
         if self.attributes['source'] and not exists(self.template):
             raise BundleError(_(
