@@ -19,6 +19,7 @@ from .exceptions import (
     NodeLockedException,
     NoSuchBundle,
     RepositoryError,
+    SkipNode,
 )
 from .group import GROUP_ATTR_DEFAULTS
 from .itemqueue import ItemQueue
@@ -344,23 +345,17 @@ class Node(object):
         OS_FAMILY_DEBIAN + \
         OS_FAMILY_REDHAT
 
-    OS_KNOWN = OS_FAMILY_BSD + OS_FAMILY_LINUX
+    OS_FAMILY_UNIX = OS_FAMILY_BSD + OS_FAMILY_LINUX
+    OS_KNOWN = OS_FAMILY_UNIX + ('kubernetes',)
 
-    def __init__(self, name, attributes=None, transport='ssh', transport_options=None):
+    def __init__(self, name, attributes=None):
         if attributes is None:
             attributes = {}
-        if transport_options is None:
-            transport_options = {}
-
-        if transport == 'ssh':
-            transport_options.setdefault(
-                'add_host_keys',
-                environ.get('BW_ADD_HOST_KEYS', False) == "1",
-            )
 
         if not validate_name(name):
             raise RepositoryError(_("'{}' is not a valid node name").format(name))
 
+        self._add_host_keys = environ.get('BW_ADD_HOST_KEYS', False) == "1"
         self._bundles = attributes.get('bundles', [])
         self._compiling_metadata = Lock()
         self._dynamic_group_lock = Lock()
@@ -369,10 +364,9 @@ class Node(object):
         self._node_metadata = attributes.get('metadata', {})
         self._ssh_conn_established = False
         self._ssh_first_conn_lock = Lock()
+        self._template_node_name = attributes.get('template_node')
         self.hostname = attributes.get('hostname', name)
         self.name = name
-        self.transport = transport
-        self.transport_options = transport_options
 
         for attr in GROUP_ATTR_DEFAULTS:
             setattr(self, "_{}".format(attr), attributes.get(attr))
@@ -568,19 +562,27 @@ class Node(object):
             ))
             return None
 
-        start = datetime.now()
+        try:
+            self.repo.hooks.node_apply_start(
+                self.repo,
+                self,
+                interactive=interactive,
+            )
+        except SkipNode as exc:
+            io.stdout(_("{x} {node}  skipped by hook ({reason})").format(
+                node=bold(self.name),
+                reason=str(exc) or _("no reason given"),
+                x=yellow("»"),
+            ))
+            return None
 
+        start = datetime.now()
         io.stdout(_("{x} {node}  {started} at {time}").format(
             node=bold(self.name),
             started=bold(_("started")),
             time=start.strftime("%Y-%m-%d %H:%M:%S"),
             x=blue("i"),
         ))
-        self.repo.hooks.node_apply_start(
-            self.repo,
-            self,
-            interactive=interactive,
-        )
 
         try:
             with NodeLock(self, interactive=interactive, ignore=force) as lock:
@@ -632,7 +634,7 @@ class Node(object):
             self.hostname,
             remote_path,
             local_path,
-            add_host_keys=self.transport_options['add_host_keys'],
+            add_host_keys=self._add_host_keys,
             wrapper_inner=self.cmd_wrapper_inner,
             wrapper_outer=self.cmd_wrapper_outer,
         )
@@ -687,6 +689,8 @@ class Node(object):
         return self.repo._metadata_for_node(self.name, partial=True)
 
     def run(self, command, data_stdin=None, may_fail=False, log_output=False):
+        assert self.os in self.OS_FAMILY_UNIX
+
         if log_output:
             def log_function(msg):
                 io.stdout("{x} {node}  {msg}".format(
@@ -697,8 +701,6 @@ class Node(object):
         else:
             log_function = None
 
-        add_host_keys = self.transport_options['add_host_keys']
-
         if not self._ssh_conn_established:
             # Sometimes we're opening SSH connections to a node too fast
             # for OpenSSH to establish the ControlMaster socket for the
@@ -708,7 +710,8 @@ class Node(object):
             # multiplexed connection.
             if self._ssh_first_conn_lock.acquire(False):
                 try:
-                    operations.run(self.hostname, "true", add_host_keys=add_host_keys)
+                    with io.job(_("{}  establishing connection...").format(bold(self.name))):
+                        operations.run(self.hostname, "true", add_host_keys=self._add_host_keys)
                     self._ssh_conn_established = True
                 finally:
                     self._ssh_first_conn_lock.release()
@@ -721,7 +724,7 @@ class Node(object):
         return operations.run(
             self.hostname,
             command,
-            add_host_keys=add_host_keys,
+            add_host_keys=self._add_host_keys,
             data_stdin=data_stdin,
             ignore_failure=may_fail,
             log_function=log_function,
@@ -729,12 +732,27 @@ class Node(object):
             wrapper_outer=self.cmd_wrapper_outer,
         )
 
+    @property
+    def template_node(self):
+        if not self._template_node_name:
+            return None
+        else:
+            target_node = self.repo.get_node(self._template_node_name)
+            if target_node._template_node_name:
+                raise RepositoryError(_(
+                    "{template_node} cannot use template_node because {node} uses {template_node} "
+                    "as template_node"
+                ).format(node=self.name, template_node=target_node.name))
+            else:
+                return target_node
+
     def upload(self, local_path, remote_path, mode=None, owner="", group="", may_fail=False):
+        assert self.os in self.OS_FAMILY_UNIX
         return operations.upload(
             self.hostname,
             local_path,
             remote_path,
-            add_host_keys=self.transport_options['add_host_keys'],
+            add_host_keys=self._add_host_keys,
             group=group,
             mode=mode,
             owner=owner,
@@ -744,24 +762,20 @@ class Node(object):
         )
 
     def verify(self, show_all=False, workers=4):
-        bad = 0
-        good = 0
+        result = []
         start = datetime.now()
 
         if not self.items:
             io.stdout(_("{x} {node}  has no items").format(node=bold(self.name), x=yellow("!")))
         else:
-            for item_status in verify_items(
-                self,
-                show_all=show_all,
-                workers=workers,
-            ):
-                if item_status:
-                    good += 1
-                else:
-                    bad += 1
+            result = verify_items(self, show_all=show_all, workers=workers)
 
-        return {'good': good, 'bad': bad, 'duration': datetime.now() - start}
+        return {
+            'good': result.count(True),
+            'bad': result.count(False),
+            'unknown': result.count(None),
+            'duration': datetime.now() - start,
+        }
 
 
 def build_attr_property(attr, default):
@@ -778,6 +792,10 @@ def build_attr_property(attr, default):
                 attr_source = "group:{}".format(group.name)
                 attr_value = getattr(group, attr)
 
+        if self.template_node:
+            attr_source = "template_node"
+            attr_value = getattr(self.template_node, attr)
+
         if getattr(self, "_{}".format(attr)) is not None:
             attr_source = "node"
             attr_value = getattr(self, "_{}".format(attr))
@@ -806,10 +824,7 @@ for attr, default in GROUP_ATTR_DEFAULTS.items():
 def verify_items(node, show_all=False, workers=1):
     items = []
     for item in node.items:
-        if (
-            not item.ITEM_TYPE_NAME == 'action' and
-            not item.triggered
-        ):
+        if not item.triggered:
             items.append(item)
         elif not isinstance(item, DummyItem):
             io.progress_advance()
@@ -841,6 +856,33 @@ def verify_items(node, show_all=False, workers=1):
                     'target': item.verify,
                 }
 
+    def handle_exception(task_id, exception, traceback):
+        node_name, bundle_name, item_id = task_id.split(":", 2)
+        io.progress_advance()
+        if isinstance(exception, NotImplementedError):
+            io.stdout(_("{x} {node}  {bundle}  {item}  (does not support verify)").format(
+                bundle=bold(bundle_name),
+                item=item_id,
+                node=bold(node_name),
+                x=cyan("?"),
+            ))
+        else:
+            # Unlike with `bw apply`, it is OK for `bw verify` to encounter
+            # exceptions when getting an item's status. `bw verify` doesn't
+            # care about dependencies and therefore cannot know that looking
+            # up a database user requires the database to be installed in
+            # the first place.
+            io.debug("exception while verifying {}:".format(task_id))
+            io.debug(traceback)
+            io.debug(repr(exception))
+            io.stdout(_("{x} {node}  {bundle}  {item}  (unable to get status, check --debug for details)").format(
+                bundle=bold(bundle_name),
+                item=item_id,
+                node=bold(node_name),
+                x=cyan("?"),
+            ))
+        return None  # count this result as "unknown"
+
     def handle_result(task_id, return_value, duration):
         io.progress_advance()
         unless_result, item_status = return_value
@@ -874,6 +916,7 @@ def verify_items(node, show_all=False, workers=1):
         tasks_available,
         next_task,
         handle_result,
+        handle_exception=handle_exception,
         pool_id="verify_{}".format(node.name),
         workers=workers,
     )
