@@ -30,7 +30,7 @@ from .metadata import (
 from .node import _flatten_group_hierarchy, Node
 from .secrets import FILENAME_SECRETS, generate_initial_secrets_cfg, SecretProxy
 from .utils import cached_property, names
-from .utils.scm import get_rev
+from .utils.scm import get_git_branch, get_git_clean, get_rev
 from .utils.dicts import hash_statedict, merge_dict
 from .utils.text import bold, mark_for_translation as _, red, validate_name
 from .utils.ui import io, QUIT_EVENT
@@ -188,7 +188,7 @@ def items_from_path(path):
     item classes globally.
     """
     if not isdir(path):
-        raise StopIteration()
+        return
     for filename in listdir(path):
         filepath = join(path, filename)
         if not filename.endswith(".py") or \
@@ -317,6 +317,10 @@ class Repository(object):
         node.repo = self
         self.node_dict[node.name] = node
 
+    @cached_property
+    def branch(self):
+        return get_git_branch()
+
     @cached_property
     def cdict(self):
         repo_dict = {}
@@ -324,6 +328,10 @@ class Repository(object):
             repo_dict[node.name] = node.hash()
         return repo_dict
 
+    @cached_property
+    def clean(self):
+        return get_git_clean()
+
     @classmethod
     def create(cls, path):
         """
@@ -513,18 +521,21 @@ class Repository(object):
                     # start messing with these objects in metadata processors. Every
                     # time we would edit one of these objects, the changes would be
                     # shared amongst multiple nodes.
-                    new_metadata = deepcopy_metadata(merge_dict(
-                        self._node_metadata_partial[node.name],
-                        node._node_metadata,
-                    ))
-                    if blame:
-                        blame_changed_paths(
+                    for source_node in (node.template_node, node):
+                        if not source_node:  # template_node might be None
+                            continue
+                        new_metadata = deepcopy_metadata(merge_dict(
                             self._node_metadata_partial[node.name],
-                            new_metadata,
-                            node_blame,
-                            "node:{}".format(node.name),
-                        )
-                    self._node_metadata_partial[node.name] = new_metadata
+                            source_node._node_metadata,
+                        ))
+                        if blame:
+                            blame_changed_paths(
+                                self._node_metadata_partial[node.name],
+                                new_metadata,
+                                node_blame,
+                                "node:{}".format(source_node.name),
+                            )
+                        self._node_metadata_partial[node.name] = new_metadata
 
             # Now for the interesting part: We run all metadata processors
             # until none of them return DONE anymore (indicating that they're
