@@ -107,6 +107,7 @@ class PathInfo(object):
             "text" in self.desc or
             self.desc in (
                           "empty",
+                          "OpenSSH ED25519 public key",
                           "OpenSSH RSA public key",
                           "OpenSSH DSA public key",
                          )
@@ -128,6 +129,18 @@ class PathInfo(object):
             result = self.node.run("sha1 -q -- {}".format(quote(self.path)))
         else:
             result = self.node.run("sha1sum -- {}".format(quote(self.path)))
+        # sha1sum adds a leading backslash to hashes of files whose name
+        # contains backslash-escaped characters – we must lstrip() that
+        return force_text(result.stdout).strip().lstrip("\\").split()[0]
+
+    @cached_property
+    def sha256(self):
+        if self.node.os == 'macos':
+            result = self.node.run("shasum -a 256 -- {}".format(quote(self.path)))
+        elif self.node.os in self.node.OS_FAMILY_BSD:
+            result = self.node.run("sha256 -q -- {}".format(quote(self.path)))
+        else:
+            result = self.node.run("sha256sum -- {}".format(quote(self.path)))
         return force_text(result.stdout).strip().split()[0]
 
     @property
